/*
 * MontyHallProblem.java
 * This codes is released under Gnu General Public License (GPL)
 *
 * Created on 09 December 2004
 */

/**
 * This class proofes the Monty Hall Problem empirically ...
 * @author  joerg.feurhake@free-penguin.org
 */
import net.unverkaeuflich.feuerhake.montyhall.*;
import java.io.File;
import java.io.FileWriter;

public class MontyHallProblem {
    
    //The Players and one auxilliary object
    MontyHall Monty;
    MontyCandidate Candidate;
    MontyDoors Doors;
    MontyResults Results;
    
    /** Creates a new instance of MontyHallProblem */
    public MontyHallProblem(boolean playRand) {
        int montyOpens = -1;
        int firstChosen = -1;
        
        //construct the show
        Doors = new MontyDoors();
        Candidate = new MontyCandidate();
        Results = new MontyResults();
        
        //wirte down what s been chosen first
        firstChosen = Candidate.getDoorChosen();
        
        Monty = new MontyHall(Doors, Candidate.getDoorChosen());
        montyOpens = Monty.openDoor();
        if(playRand){
            Candidate.changeDoorsRand(montyOpens);
        }
        else{
            Candidate.changeDoors(montyOpens);
        };
        
        if(Candidate.getDoorChosen() == Doors.getCarDoor()){
            Candidate.setWon(true);
        };
        
        //note the results
        Results.setCarDoor(Doors.getCarDoor());
        Results.setChanged(playRand);
        Results.setFirstChosen(firstChosen);
        Results.setMontyOpened(montyOpens);
        Results.setScndChosen(Candidate.getDoorChosen());
        Results.setWon(Candidate.getWon());
        
        //check for obvious program errors
        if(montyOpens == Doors.getCarDoor()){
            System.out.println("DEBUG::algorithm broken. Monty opened wrong door ...");
            //System.exit(-1);
        };
        
        if(firstChosen == Candidate.getDoorChosen() && playRand == false){
            System.out.println("DEBUG::algorithm broken. Candidate didnt change doors...");
            //System.exit(-1);
        };
        
        //clean up after the show
        Monty = null;
        Doors = null;
        Candidate = null;
        //System.gc();
    }
    
    /**
     * The main method
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        
        int trials = -1;
        int k = 0;
        int l = 0;
        int n = 0;
        boolean noOut = false;
        StringBuffer resultString = new StringBuffer("Nr, Switched, DoorFirstChosen, DoorMontyOpened, DoorScndChosen, CarDoor, WonGame\n");
        
        //no args - print usage message
        if(args.length == 0){
            System.out.println("Please give as first argument the number of trials monty should play.\nThe second argument is optional. \nAs second argument you may pass the name of a .csv file \nto write the trial results to.\n\nexample: 'java -jar montyHallProblem.jar 1000 /home/me/out.csv' \nor java -jar montyHallProblem.jar 1000000 \n\nFor really large numbers of trials you may want to disable output of single trials\nYou can do this by typing:\n\njava -jar montyhallproblem.jar 10000000 NoOutput");
            System.exit(-1);
        }
        if(args.length == 2 && args[1].equals("NoOutput"))
            noOut = true;
        
        //the number of events
        trials = new Integer(args[0]).intValue();
        
        //true if the candidate should rechoose randomly
        boolean playRand = true;
        
        //play the game 'trials' times
        for(int j=0; j<2; j++){
            l=k;
            k=0;
            for(int i=0; i<trials; i++){
                MontyHallProblem mhp = new MontyHallProblem(playRand);
                if(mhp.getResults().getWon() == true)
                    k++;
                n++;
                if(!noOut)
                    resultString= resultString.append(n+", "+mhp.getResults().getChanged()+", "+mhp.getResults().getFirstChosen()+", "+mhp.getResults().getMontyOpened()+", "+mhp.getResults().getScndChosen()+", "+mhp.getResults().getCarDoor()+", "+mhp.getResults().getWon()+"\n");
                mhp = null;
                
            };
            playRand = false;
        };
        
        //what to do with the results?
        
        if(args.length == 1){
            //no file - just write to stdout
            if(!noOut)
                System.out.println(new String(resultString));
            
        }
        else{
            //try to write to given file
            if(!noOut){
                try{
                    File out = new File(args[1]);
                    FileWriter fw = new FileWriter(out);
                    fw.write(new String(resultString));
                    fw.flush();
                }
                catch(Exception e){
                    System.out.println("Couldn't write file...");
                    e.printStackTrace();
                };
            };
        };
        //print out some very general infomation about the experiment
        System.out.println("In "+trials+" Versuchen hat der Kandidat beim strikten wechseln "+k+" mal gewonnen. Beim spielen mit zufaelligem neuwaehlen hat er "+l+" mal gewonnen.");
        
    }
    /**
     * this method is here to pass the results of each trial to the main method
     */
    private MontyResults getResults(){
        return Results;
    }
}
